
import os
import shutil
import contextlib

from datetime import date
from functools import partial

from odoo.tests import common
from odoo.addons.saas_worker.compat import http
from odoo.addons.website import tools as website_tools


@contextlib.contextmanager
def MockRequest(env, *args, **kwargs):
    # extended MockRequest with passthrough for redirect/render/not_found
    with website_tools.MockRequest(env, *args, **kwargs) as request:
        request.configure_mock(
            redirect=partial(http.Request.redirect, request),
            render=partial(http.Request.render, request),
            not_found=partial(http.Request.not_found, request),
        )
        yield request


def _init_loempia(cls):
    cls.website = cls.env['website'].browse(1)

    cls.maintainer = cls.env.ref('loempia.user_tpl')
    cls.maintainer.write({'oauth_uid': "dummy_oauth_uid"})

    cls.customer = cls.env.ref('base.demo_user0')
    cls.customer_bis = cls.env.ref('base.template_portal_user_id')

    cls.functional = cls.env.ref('base.user_demo')

    cls.manager = cls.env.ref('base.user_admin')
    cls.manager.write({'oauth_uid': "dummy_manager_oauth_uid"})


class TestLoempiaData(common.TransactionCase):
    @classmethod
    def setUpClass(self):
        super(TestLoempiaData, self).setUpClass()
        _init_loempia(self)

class TestLoempia(TestLoempiaData):

    @classmethod
    def setUpClass(self):
        super(TestLoempia, self).setUpClass()

        self.repositories = self._create_repos({"loempia.series_13": 13.0, "loempia.series_12": 12.0, "loempia.series_11": 11.0})
        self.repo_11 = self.repositories.filtered(lambda r: r.series_id.id == self.env.ref('loempia.series_11').id)
        self.repo_12 = self.repositories.filtered(lambda r: r.series_id.id == self.env.ref('loempia.series_12').id)
        self.repo_13 = self.repositories.filtered(lambda r: r.series_id.id == self.env.ref('loempia.series_13').id)

        # Module A  -- V12 purchased
        self.module_11_A = self._create_module("Module_A", self.repo_11, 1)
        self.module_12_A = self._create_module("Module_A", self.repo_12, 1)
        self.module_13_A = self._create_module("Module_A", self.repo_13, 2)  # Module in v13 are considered as another product
        self.template_A = self.module_11_A.template_id  # All the pre-created module shares the same template
        self.purchase_A = self._create_confirmed_purchase(self.module_11_A)

        # Module B -- V12 purchased, V11 rated 4 times
        self.module_11_B = self._create_module("Module_B", self.repo_11, 3)
        self.module_12_B = self._create_module("Module_B", self.repo_12, 3)
        self.module_13_B = self._create_module("Module_B", self.repo_13, 4)  # Module in v13 are considered as another product
        self.template_B = self.module_11_B.template_id  # All the pre-created module shares the same template
        self.purchase_B = self._create_confirmed_purchase(self.module_11_B)
        self.ratings_11_B = self._create_ratings(self.module_11_B)

        # Module Enterprise
        self.enterprise_repo = self._create_repos({"loempia.series_13": 13.0}, enterprise_repo=True)
        self.module_enterprise = self._create_module("Module_odoo", self.enterprise_repo, 5, price_single=0, approved=True)

        # Free module and its dependecies
        self.module_free_dpd = self._create_module("Module_free_dpd", self.repo_13, 6)
        self.module_free = self._create_module("Module_free", self.repo_13, 0, price_single=0, depend_ids=[(4, self.module_free_dpd.id, 0)])
        self.purchase_free_dpd = self._create_confirmed_purchase(self.module_free_dpd)

    @classmethod
    def tearDownClass(self):
        for repo in self.repositories:
            if os.path.exists(repo.path):
                shutil.rmtree(repo.path)
        super(TestLoempia, self).tearDownClass()

    @classmethod
    def _create_module(self, module_name, repo_id, product_id, price_single=10.0, approved=False, depend_ids=None):
        template = self.env['loempia.module.template'].search([('name', '=', module_name), ('maintainer_id', '=', repo_id.maintainer_id.id)], limit=1)
        return self.env['loempia.module'].create({
            'name': module_name,
            'version': '1.0',
            'license': 'LGPL-3',
            'currency_id': self.env.ref('base.EUR').id,
            'price_single': price_single,
            'repo_id': repo_id.id,
            'template_id': template.id if template else self.env['loempia.module.template'].create(
                {
                    'name': module_name,
                    'maintainer_id': repo_id.maintainer_id.id,
                }).id,
            'product_id': product_id,
            'approved': approved,
            'depend_ids': [] if not depend_ids else depend_ids,
        })

    @classmethod
    def _create_repos(self, series, enterprise_repo=False):
        repositories = self.env['loempia.repo']
        for k, v in series.items():
            repositories |= self.env['loempia.repo'].create({
                'series_id': self.env.ref(k).id,
                'url': 'ssh://git@github.com/ndeodoo/essai.git#%s' % str(v) if not enterprise_repo else 'ssh://git@github.com/odoo/enterprise.git#%s' % str(v),
                'vcs': 'git',
                'maintainer_id': self.maintainer.id,
                'state': 'active',
            })
        return repositories

    @classmethod
    def _create_confirmed_purchase(self, module_id):
        return self.env['loempia.module.purchase'].create({
            'order_id': 1,
            'order_name': 'SO0001',
            'module_id': module_id._get_commercial_series_module_id().id,
            'template_id': module_id.template_id.id,
            'product_id': module_id._get_commercial_series_module_id().product_id,
            'date_order': date.today(),
            'quantity': 1,
            'price_unit': module_id._get_commercial_series_module_id().price_single,
            'user_id': self.customer.id,
            'state': 'confirmed',
        })

    @classmethod
    def _create_ratings(self, module_id):
        # It should never been created like that
        # as one user will not be able to let multiple rating
        for i in range(4):
            self.env['loempia.module.rating'].create({
                'module_id': module_id.id,  # To comment both module
                'template_id': module_id.template_id.id,
                'rating': 4,
                'subject': 'test',
                'body': 'test',
                'author_id': self.functional.partner_id.id,
            })
